! module for handling different sets of coordinates (cartesian, orbital...)
module coordinates

  implicit none

  double precision,parameter:: PI = 3.141592653589793238462643d0
  double precision,parameter:: DPI = 2.d0*PI

contains
!***************************************************************************************

  ! Performs the change of coordinates from Cartesian elements to Keplerian coordinates
  !
  subroutine CarToKep(mu, xcar, xkep)

     implicit none

     double precision,intent(in):: mu         ! G*(m0+mi)
     double precision, intent(in):: xcar(6)
     double precision, intent(out):: xkep(6)

     double precision:: x(3), v(3), angm(3), lenz(3), norb(3)
     double precision:: v2, r, em, ener, nm
     double precision:: e3(3), ascnod(3), aux(3)
     double precision:: aa, ecc, inc, omnod, omega, u, ell
     double precision:: cosw, sinw, cosu, sinu

     !------------------------------------------------
     x  = xcar(1:3)                                                 ! cartesian position
     v  = xcar(4:6)                                                 ! cartesian velocity
     v2 = prscal(v,v)                                               
     r  = sqrt(prscal(x,x))                                          
     !------------------------------------------------
     call prvec(x,v,angm)                                           ! angular momentum vector 
     em   = sqrt(prscal(angm,angm))
     call prvec(v,angm,lenz)                                         
     lenz = lenz/mu-x/r                                             ! Lenz vector
     norb = angm/em                                                 ! orbit normal unit vector
     ener = v2/2-mu/r                                               ! energy
     !------------------------------------------------
     aa  = -mu/(2.d0*ener)                                          ! semi-amjor axis
     nm  = sqrt(mu/aa**3)                                           ! mean motion
     ecc = sqrt(prscal(lenz,lenz))                                  ! eccentricity
     inc = acos(norb(3))                                            ! inclination
     !------------------------------------------------
     e3 = 0.d0                                                      ! e3 versor
     e3(3) = 1.d0
     call prvec(e3,angm,ascnod)
     ascnod = ascnod/sqrt(prscal(ascnod,ascnod))                    ! ascending node unit vector
     omnod = datan2(ascnod(2),ascnod(1))                            ! longitude of the node
     !------------------------------------------------
     call prvec(ascnod,lenz/ecc,aux)
     cosw = prscal(lenz,ascnod)/ecc
     sinw = sqrt(prscal(aux,aux))
     ! WARNING: check the real sign of sinw
     if ((inc.lt.PI/2.d0.and.aux(3).lt.0).or.(inc.gt.PI/2.d0.and.aux(3).gt.0)) then
        sinw=-sinw
     endif
     omega = datan2(sinw,cosw)                                      ! argument of the pericenter
     !------------------------------------------------
     cosu = 1.d0/ecc*(1.d0-r/aa)
     sinu = prscal(x,v)/(aa**2*ecc*nm)
     u = datan2(sinu,cosu)                                       ! eccentric anomaly
     ell = u-ecc*sinu                                            ! mean anomaly
     !------------------------------------------------
     xkep(1) = aa                             ! semi-amjor axis
     xkep(2) = ecc                            ! eccentricity
     xkep(3) = inc                            ! inclination
     xkep(4) = modulo(omega,DPI)              ! pericenter argument
     xkep(5) = modulo(omnod,DPI)              ! longitude of the node
     xkep(6) = modulo(ell,DPI)                ! mean anomaly 

  end subroutine CarToKep

!***************************************************************************************

  subroutine KepToCar(mu, kep, car) 
     double precision:: mu
     double precision,intent(in) :: kep(6)
     double precision,intent(out) :: car(6)
     !end interface
     double precision :: ell,omeg,Omnod,inc
     double precision :: ci,si,co,so,con,son
     double precision,DIMENSION(3,3) :: R_om,R_i,R_Omn
     double precision,DIMENSION(3,3) :: RiRom,ROmnRi
     double precision,DIMENSION(3,3) :: Rot
     double precision :: ecc,aa,nm
     double precision :: pos(3)
     double precision :: u,sinu,cosu,dotu
     double precision :: dposdu(3),vel(3)
     double precision :: normpos
     
     inc   = kep(3)
     omeg  = kep(4)
     Omnod = kep(5)
     ell   = kep(6)
     
      ! Rotation matrix of angle omega
     co = cos(omeg)
     so = sin(omeg)
     R_om = 0.d0
     R_om(1,1) = co
     R_om(1,2) =-so
     R_om(2,1) = so
     R_om(2,2) = co
     R_om(3,3) = 1.d0
 
     ! Rotation matrix of angle Inclination
     ci = cos(inc)
     si = sin(inc)
     R_i = 0.d0
     R_i(1,1) = 1.d0
     R_i(2,2) = ci
     R_i(2,3) =-si
     R_i(3,2) = si
     R_i(3,3) = ci
        
     ! R_i*R_om
     RiRom = MATMUL(R_i,R_om)
        
     ! Rotation matrix of angle omega
     con = cos(Omnod)
     son = sin(Omnod)
     R_Omn = 0.d0
     R_Omn(1,1) = con
     R_Omn(1,2) =-son
     R_Omn(2,1) = son
     R_Omn(2,2) = con
     R_Omn(3,3) = 1.d0
       
     ROmnRi = MATMUL(R_Omn,R_i)
     ! R_Omn*R_i*R_om
     Rot = MATMUL(R_Omn,RiRom)
            
     ecc = kep(2)
     aa = kep(1)
     CALL keplereq(ell,ecc,u)
     cosu = cos(u)
     sinu = sin(u)
       
     pos(1) = aa*(cosu - ecc)
     pos(2) = aa*sqrt(1.d0-ecc**2)*sinu 
     pos(3) = 0.d0 
 
     dposdu(1) = aa*(-sinu)
     dposdu(2) = aa*sqrt(1.d0-ecc**2)*cosu
     dposdu(3) = 0.d0
         
     normpos = sqrt(prscal(pos(1:3),pos(1:3)))
     nm = sqrt(mu/aa**3)
     dotu = nm*aa/normpos
     vel(1) = dposdu(1)*dotu
     vel(2) = dposdu(2)*dotu
     vel(3) = 0.d0
        
     car(1:3) =MATMUL(Rot,pos(1:3))
     car(4:6) =MATMUL(Rot,vel(1:3))    
 
  end subroutine KepToCar

!***************************************************************************************

  subroutine keplereq(ell,ecc,u)
    double precision, intent(in) :: ell,ecc
    double precision, intent(out) :: u
    ! end interface
    integer :: j
    integer,parameter :: jmax=100
    double precision :: ukp1,uk, elle

    uk=PI
    elle = mod(ell,DPI)
    do j=1,jmax
       ukp1 = uk + (elle-uk+ecc*sin(uk))/(1.d0-ecc*cos(uk))
       if(abs(ukp1-uk).le.1.d-14)then
          exit
       endif
       uk=ukp1
    enddo
    u = ukp1
  end subroutine keplereq

!***************************************************************************************

  ! scalar product
  function prscal(a,b)

     implicit none

     double precision:: prscal
     double precision, intent(in):: a(3), b(3)

     prscal=a(1)*b(1)+a(2)*b(2)+a(3)*b(3)

  end function prscal

!***************************************************************************************

  ! vector product
  subroutine prvec(a,b,c)

     implicit none

     double precision, intent(in):: a(3), b(3)
     double precision, intent(out):: c(3)

     c(1) = a(2)*b(3)-a(3)*b(2)
     c(2) = a(3)*b(1)-a(1)*b(3)
     c(3) = a(1)*b(2)-a(2)*b(1)

  end subroutine prvec

!***************************************************************************************

end module coordinates
