! Simple program for the propagation of the Kepler problem
! course of Elementi di Meccanica Celeste (UniPi)
program keplerEMC

  use parameters

  implicit none

  character(256):: hamParamFile ! namelist containing the physical parameters
  character(256):: intParamFile ! namelist containing the parameters of the numerical integration
  character(256):: outFile      ! output file

  type(hamparam):: Hp  ! Physical parameters
  type(intparam):: Ip  ! parameters of the numerical integration

  double precision:: X(6),XK(6) ! state vector
  double precision:: t ! time
  integer:: Nstep, Nreg, Nreg2, Nreg3

!---------------------------------------------------------------------------------------------
! Reading and computing the parameters

  if(COMMAND_ARGUMENT_COUNT() >= 3) then
     call GET_COMMAND_ARGUMENT(1,hamParamFile)
     call GET_COMMAND_ARGUMENT(2,intParamFile)
     call GET_COMMAND_ARGUMENT(3,outFile)
  else
     write(*,*) "Use: <program name> <parameterFile.nml> <initialConditions.nml> <outFile.dat>"
     stop
  endIf

  ! computing the physical parameters
  call readHamParam(hamParamFile,trim(outFile)//"_ham.nml",Hp)

  ! computing the parameters of the numerical integration
  call initInteg(intParamFile,trim(outFile)//"_int.nml", Hp,Ip, X)

!---------------------------------------------------------------------------------------------
! Initializations

  t = Ip%t0
  Nstep = 0 ! number of timesteps realised

  ! Opening and writing the header of the output file with orbital elements
  open(newunit=Nreg,file=trim(outFile),status='REPLACE',action='WRITE') ! opening the output file
  write(Nreg,'(A)') "# a is the semi-major axis"
  write(Nreg,'(A)') "# e is the eccentricity"
  write(Nreg,'(A)') "# I is the inclination"
  write(Nreg,'(A)') "# w is the argument of pericenter"
  write(Nreg,'(A)') "# O is the longitude of ascending node"
  write(Nreg,'(A)') "# l is the mean anomaly"
  write(Nreg,'(A)') "# The angles are expressed in radians."
  write(Nreg,'(A)') "# "
  write(Nreg,'(A,6(I0,A))') "# t ; a, e, I, w, O, l;"
  
  ! Opening and writing the header of the output file with cartesian coordinates
  open(newunit=Nreg2,file=trim(outFile)//"_car",status='REPLACE',action='WRITE')  ! opening the output file
  write(Nreg2,'(A,6(I0,A))') "# t ; x, y, z, vx, vy, vz;"
  
  ! Opening and writing the header of the output file with cartesian coordinates in the orbital plane
  open(newunit=Nreg3,file=trim(outFile)//"_plan",status='REPLACE',action='WRITE') ! opening the output file
  write(Nreg3,'(A,6(I0,A))') "# t ; x, y, z, vx, vy, vz;"

!---------------------------------------------------------------------------------------------
! Computation of the solution of the Kepler problem from t0 to t0+Tspan

  call CarToKep(Hp%mu,X,XK)
  call printFunc(XK,t) ! saving the initial state
  do Nstep=1,INT(Ip%Tspan/Ip%Dt)
     t=t+Ip%Dt
     XK(6)=XK(6)+sqrt(Hp%mu/XK(1)**3)*Ip%Dt
     call printFunc(XK,t) ! saving the initial state
  enddo

!---------------------------------------------------------------------------------------------

  close(Nreg)  ! closing the output file
  close(Nreg2) ! closing the output file 2
  close(Nreg3) ! closing the output file 3

!---------------------------------------------------------------------------------------------
contains
!---------------------------------------------------------------------------------------------

  ! Subroutine used after each step for saving the results.
  ! 
  subroutine printFunc(XK,tt)
     double precision,intent(in):: XK(:), tt

     double precision:: XC(6),XKmod(6)

     if( mod(Nstep,Ip%Nsave) == 0 ) then

        call KepToCar(Hp%mu, XK, XC)

        ! output in keplerian elements        
        write(Nreg,'(ES22.15,*(1X,ES22.15))') tt, XK
        ! output in cartesian coordinates
        write(Nreg2,'(ES22.15,*(1X,ES22.15))') tt, XC

        XKmod=XK
        XKmod(3)=0.d0
        XKmod(4)=0.d0
        XKmod(5)=0.d0
        call KepToCar(Hp%mu, XKmod, XC)
        ! output in cartesian coordinates using the orbital plane
        write(Nreg3,'(ES22.15,*(1X,ES22.15))') tt, XC

     endIf

  end subroutine printFunc

!---------------------------------------------------------------------------------------------

end program keplerEMC
