! module for handling input files and define physical and integration parameters
module parameters

  use coordinates
  implicit none

  !-------------------------------------------------------------------------
  ! Physical parameters
  ! 
  type hamparam

     double precision:: mu                     ! G*Mtot

  end type hamparam
!-------------------------------------------------------------------------
  ! Parameters of the numerical integration
  ! 
  type intparam

     double precision:: t0    ! initial time
     double precision:: tSpan ! duration
     double precision:: Dt    ! time step
     integer:: Nsave          ! number of timesteps between each save

  end type intparam

!-------------------------------------------------------------------------
contains
!***************************************************************************************

  ! Read the physical parameters from an input file and rewrite them in an output file.
  ! All the parameters should have the same system of units (e.g. au, years, Sun-mass).
  ! 
  subroutine readHamParam(ficIn,ficOut,Hp)
     character(len=*),intent(in):: ficIn, ficOut
     type(hamparam),intent(out):: Hp

     double precision:: mu            ! G*Mtot

     integer:: Nreg

     namelist /physicalparams/ mu

     !-----------------------------------------------------------------------
     ! Reading the parameters
     open(newunit=Nreg,file=trim(ficIn),status="OLD",action="READ")
     read(Nreg,physicalparams)
     close(Nreg)

     ! Writing the parameters
     open(newunit=Nreg,file=trim(ficOut),status="REPLACE",action="WRITE")

     write(Nreg,'(A)')              "&physicalparams"
     write(Nreg,'(A,ES22.15,A)')    " mu = " , mu,","
     write(Nreg,'(A)') "/"

     close(Nreg)
     !-----------------------------------------------------------------------

     Hp%mu = mu

  end subroutine readHamParam

!***************************************************************************************

  ! Read the parameters of the numerical integration and the initial conditions in an
  ! input file, and rewrite them in an output file.
  ! 
  subroutine initInteg(ficIn,ficOut, Hp,Ip, Z0)
     character(len=*),intent(in):: ficIn, ficOut
     type(hamparam),intent(in):: Hp
     type(intparam),intent(out):: Ip
     double precision,intent(out):: Z0(6)


     double precision:: t0    ! initial time
     double precision:: tSpan ! duration
     double precision:: Dt    ! time step
     integer:: Nsave          ! number of timesteps between each save

     double precision:: X(6)

     integer:: Nreg, i

     namelist /integrationparam/ t0, tSpan, Dt, Nsave, X

     !-----------------------------------------------------------------------
     ! Reading the parameters
     open(newunit=Nreg,file=trim(ficIn),status="OLD",action="READ")
     read(Nreg,integrationparam)
     close(Nreg)

     ! Writing the parameters
     open(newunit=Nreg,file=trim(ficOut),status="REPLACE",action="WRITE")

     write(Nreg,'(A)')              "&integrationparam"
     write(Nreg,'(A,ES22.15,A)')    "     t0 = " , t0,","
     write(Nreg,'(A,ES22.15,A)')    "  tSpan = " , tSpan,","
     write(Nreg,'(A,ES22.15,A)')    "     Dt = " , Dt,","
     write(Nreg,'(A,I0,A)')         "  Nsave =  ", Nsave,","
     write(Nreg,'(A,*(ES22.15,A))') "      X = " , (X(i),",",i=1,6)
     write(Nreg,'(A)') "/"

     close(Nreg)

     ! initial conditions
     Z0=X

     !-----------------------------------------------------------------------

     ! Parameters of the numerical integration
     Ip%t0      = t0
     Ip%tSpan   = tSpan
     Ip%Dt      = Dt
     Ip%Nsave   = Nsave

     !-----------------------------------------------------------------------

  end subroutine initInteg

!***************************************************************************************
end module parameters
